/*!
 * \file    useif.h
 * \brief   Template meta-programming SFINAE helpers
 *
 * Copyright (C) 2018 Christos Choutouridis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __utl_meta_useif_h__
#define __utl_meta_useif_h__

#include <utl/core/impl.h>
#include <utl/meta/logical.h>
#include <utl/meta/sfinae.h>

/*!
 * \ingroup meta
 * \defgroup sfinae
 * conditional use support header. This is a SFINAE wrapper
 */
//! @{
namespace utl {
namespace meta {

   //! If same type resolves to _Ret, else SFINAE
   template <typename _T1, typename _T2, typename _Ret =_T1>
   using use_if_same_t = type_<
         enable_if<
            same_<_T1, _T2>::value, _Ret
         >
   >;
   //! If not same type resolves to _Ret, else SFINAE
   template <typename _T1, typename _T2, typename _Ret =_T1>
   using use_if_not_same_t = type_<
         enable_if<
            !same_<_T1, _T2>::value, _Ret
         >
   >;
   //! If any type (_T1 or _T2) type resolves to _Ret, else to SFINAE
   template <typename _T1, typename _T2, typename _Ret =_T1>
   using use_if_any_t = type_<
         enable_if<
            or_<_T1, _T2>::value, _Ret
         >
   >;

   //! If both type (_T1 and _T2) type resolves to _Ret, else to SFINAE
   template <typename _T1, typename _T2, typename _Ret =_T1>
   using use_if_both_t = type_<
         enable_if<
            and_<_T1, _T2>::value, _Ret
         >
   >;

}}

//! @}

#endif /* __utl_meta_useif_h__ */
